import 'dart:convert';
import 'dart:typed_data';
import 'package:crypto/crypto.dart';
import 'package:eudcc/eudcc.dart';
import 'package:typed_data/typed_data.dart' as typed;

import 'package:flutter/services.dart';
import 'package:asn1lib/asn1lib.dart';
import 'package:x509b/x509.dart' as x509;
import 'package:ninja/ninja.dart' as ninja;
import 'package:ninja/padder/mgf/mgf.dart';

class Trust {
  final x509.PublicKey pk;
  final dynamic kid;

  Trust(this.pk, this.kid);

  CheckResult verify(
      dynamic alg, typed.Uint8Buffer data, typed.Uint8Buffer sig) {
    bool Function(Uint8List data, x509.Signature signature) verify =
        (Uint8List data, x509.Signature signature) {
      return false;
    };

    if (pk is x509.EcPublicKey) {
      var mapping = {
        -7: x509.algorithms.signing.ecdsa.sha256,
        -35: x509.algorithms.signing.ecdsa.sha384,
        -36: x509.algorithms.signing.ecdsa.sha512,
      };
      if (!mapping.containsKey(alg)) {
        return CheckResult.unknownAlgorithm;
      } else {
        verify = pk.createVerifier(mapping[alg]!).verify;
      }
    } else if (pk is x509.RsaPublicKey) {
      var mapping = {
        -7: x509.algorithms.signing.rsa.sha256,
        -35: x509.algorithms.signing.rsa.sha384,
        -36: x509.algorithms.signing.rsa.sha512,
      };

      if (alg == -37) {
        ninja.RsaSsaPssVerifier ninv = ninja.RsaSsaPssVerifier(
            hasher: sha256, mgf: Mgf1(hasher: sha256), saltLength: 32);

        var npk = ninja.RSAPublicKey((pk as x509.RsaPublicKey).modulus,
            (pk as x509.RsaPublicKey).exponent);
        verify = (Uint8List d, x509.Signature s) {
          return ninv.verify(npk, s.data, d);
        };
      } else if (!mapping.containsKey(alg)) {
        return CheckResult.unknownAlgorithm;
      } else {
        verify = pk.createVerifier(mapping[alg]!).verify;
      }
    } else {
      return CheckResult.unknownAlgorithm;
    }

    var valid = verify(
        data.buffer.asUint8List(), x509.Signature(sig.buffer.asUint8List()));
    return valid ? CheckResult.valid : CheckResult.invalid;
  }
}

class TrustStore {
  static final TrustStore _singleton = TrustStore._internal();

  factory TrustStore() {
    return _singleton;
  }

  TrustStore._internal();

  final Map<String, Trust> _data = {};

  Future<void> load() async {
    var m =
        jsonDecode(await rootBundle.loadString("specs/trust/trust-all.json"));
    m.forEach((item) {
      var kid = item["kid"];
      var asn = base64Decode(item["publicKey"]);

      var p = ASN1Parser(asn);
      var o = p.nextObject();
      if (o is! ASN1Sequence) {
        throw FormatException('Expected SEQUENCE, got ${o.runtimeType}');
      }
      // Public Key file (PKCS#8)
      var pub = x509.SubjectPublicKeyInfo.fromAsn1(o);
      _data[kid] = Trust(pub.subjectPublicKey, kid);
    });
    return;
  }

  bool containsKey(String kid) {
    return _data.containsKey(kid);
  }

  Trust? get(String kid) {
    return _data[kid];
  }
}
