import 'dart:convert';
import 'dart:io';

import 'package:eudcc/certificates/certificate.dart';
import 'package:eudcc/eudcc.dart';
import 'package:flutter_test/flutter_test.dart';

_loadJson(String fname) async {
  final file = File(fname);
  return jsonDecode(await file.readAsString());
}

void main() {
  test('parse dob', () {
    var vectors = {
      "1979-04-14": {"year": 1979, "month": 4, "day": 14, "repr": "1979-04-14"},
      "1901-08": {"year": 1901, "month": 8, "day": null, "repr": "1901-08-XX"},
      "1939": {"year": 1939, "month": null, "day": null, "repr": "1939-XX-XX"},
      "": {"year": null, "month": null, "day": null, "repr": "XXXX-XX-XX"},
    };

    for (String vector in vectors.keys) {
      var dob = BirthDate.fromJsonKey(vector);
      var r = vectors[vector];
      expect(true, r!.isNotEmpty);
      expect(dob.day, r["day"]);
      expect(dob.month, r["month"]);
      expect(dob.year, r["year"]);
      expect(dob.toString(), r["repr"]);
    }

    expect(BirthDate.fromJsonKey("1979-04-05").toDateTime(),
        DateTime.utc(1979, 4, 5));
    expect(
        BirthDate.fromJsonKey("1979").toDateTime(), DateTime.utc(1979, 0, 0));
  });
  test('parse certificate empty', () {
    var json = {
      "dob": "1979-04-14",
      "nam": {
        "gn": "Isolde Erika",
        "gnt": "ISOLDE<ERIKA",
        "fn": "Musterfrau-Gößinger Müller",
        "fnt": "MUSTERFRAU<GOESSINGER<MUELLER"
      },
      "ver": "1.3.0"
    };
    var cert = Certificate.fromJson(json);
    expect(cert.version, '1.3.0');
    expect(cert.dateOfBirth.toString(), "1979-04-14");
    expect(cert.holder.givenName, "Isolde Erika");
  });
  test('parse certificate vaccine', () {
    var json = {
      "dob": "1979-04-14",
      "nam": {
        "gn": "Isolde Erika",
        "gnt": "ISOLDE<ERIKA",
        "fn": "Musterfrau-Gößinger Müller",
        "fnt": "MUSTERFRAU<GOESSINGER<MUELLER"
      },
      "ver": "1.3.0",
      "v": [
        {
          "tg": "840539006",
          "vp": "1119349007",
          "mp": "EU/1/20/1528",
          "ma": "ORG-100030215",
          "dn": 1,
          "sd": 2,
          "dt": "2021-03-28",
          "co": "CZ",
          "is": "Ministry of Health of the Czech Republic",
          "ci": "URN:UVCI:01:AT:10807843F94AEE0EE5093FBC254BD813#B"
        }
      ]
    };
    var cert = Certificate.fromJson(json);
    expect(cert.version, '1.3.0');
    expect(cert.dateOfBirth.toString(), "1979-04-14");
    expect(cert.holder.givenName, "Isolde Erika");
    expect(cert.vaccines?.length, 1);
  });
  test('parse certificate test', () {
    var json = {
      "dob": "1979-04-14",
      "nam": {
        "gn": "Isolde Erika",
        "gnt": "ISOLDE<ERIKA",
        "fn": "Musterfrau-Gößinger Müller",
        "fnt": "MUSTERFRAU<GOESSINGER<MUELLER"
      },
      "ver": "1.3.0",
      "t": [
        {
          "tg": "840539006",
          "tt": "LP217198-3",
          "ma": "344",
          "sc": "2021-08-20T12:03:12+02:00",
          "tr": "260415000",
          "tc": "Test centre west region 245",
          "co": "CZ",
          "is": "North-West region health authority",
          "ci": "URN:UVCI:01:AT:10807843F94AEE0EE5093FBC254BD813#B"
        }
      ]
    };
    var cert = Certificate.fromJson(json);
    expect(cert.version, '1.3.0');
    expect(cert.dateOfBirth.toString(), "1979-04-14");
    expect(cert.holder.givenName, "Isolde Erika");
    expect(cert.tests?.length, 1);
  });
  test('parse certificate recovery', () {
    var json = {
      "dob": "1979-04-14",
      "nam": {
        "gn": "Isolde Erika",
        "gnt": "ISOLDE<ERIKA",
        "fn": "Musterfrau-Gößinger Müller",
        "fnt": "MUSTERFRAU<GOESSINGER<MUELLER"
      },
      "ver": "1.3.0",
      "r": [
        {
          "tg": "840539006",
          "fr": "2021-05-18",
          "co": "UNHCR",
          "is": "Central University Hospital",
          "df": "2021-05-29",
          "du": "2021-11-14",
          "ci": "URN:UVCI:01:NL:187/37512422923"
        }
      ]
    };
    var cert = Certificate.fromJson(json);
    expect(cert.version, '1.3.0');
    expect(cert.dateOfBirth.toString(), "1979-04-14");
    expect(cert.holder.givenName, "Isolde Erika");
    expect(cert.recoveries?.length, 1);
  });
  test('parse "ehn-dcc-schema/examples/recovery/simple.json"', () async {
    TestWidgetsFlutterBinding.ensureInitialized();
    await EUDigitalCovidCertificateFactory().loadDataset(null);

    final cert = Certificate.fromJson(
        await _loadJson('specs/ehn-dcc-schema/examples/recovery/simple.json'));
    expect(cert.dateOfBirth.toString(), "1964-01-01");
    expect(cert.recoveries?.length, 1);
    expect(cert.recoveries![0].country.key, "NL");
    expect(cert.recoveries![0].country.displayName, "Netherlands");
  });
  test('parse "ehn-dcc-schema/examples/test/simple-naat.json"', () async {
    TestWidgetsFlutterBinding.ensureInitialized();
    await EUDigitalCovidCertificateFactory().loadDataset(null);

    final cert = Certificate.fromJson(
        await _loadJson('specs/ehn-dcc-schema/examples/test/simple-naat.json'));
    expect(cert.dateOfBirth.toString(), "1964-01-01");
    expect(cert.tests?.length, 1);
    expect(cert.tests![0].country.key, "NL");
    expect(cert.tests![0].country.displayName, "Netherlands");
    expect(cert.tests![0].facility, "Example Test Corp, Big City");
    expect(cert.tests![0].name, "ELITechGroup, SARS-CoV-2 ELITe MGB® Kit");
  });
  test('parse "ehn-dcc-schema/examples/test/simple-rat.json"', () async {
    TestWidgetsFlutterBinding.ensureInitialized();
    await EUDigitalCovidCertificateFactory().loadDataset(null);

    final cert = Certificate.fromJson(
        await _loadJson('specs/ehn-dcc-schema/examples/test/simple-rat.json'));
    expect(cert.dateOfBirth.toString(), "1964-01-01");
    expect(cert.tests?.length, 1);
    expect(cert.tests![0].country.key, "NL");
    expect(cert.tests![0].country.displayName, "Netherlands");
    expect(cert.tests![0].facility, "Example Test Corp, Big City");
    expect(cert.tests![0].manufacturer!.key, "532");
  });
  test('parse "ehn-dcc-schema/examples/vaccination/simple.json"', () async {
    TestWidgetsFlutterBinding.ensureInitialized();
    await EUDigitalCovidCertificateFactory().loadDataset(null);

    final cert = Certificate.fromJson(await _loadJson(
        'specs/ehn-dcc-schema/examples/vaccination/simple.json'));
    expect(cert.dateOfBirth.toString(), "1964-01-01");
    expect(cert.vaccines?.length, 1);
    expect(cert.vaccines![0].country.key, "NL");
    expect(cert.vaccines![0].country.displayName, "Netherlands");
    expect(cert.vaccines![0].manufacturer.key, "ORG-100031184");
  });
}
